@@ -1,5 +1,5 @@
 /*
-  Copyright 2011-2016 David Robillard <http://drobilla.net>
+  Copyright 2011-2020 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -14,8 +14,11 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-#include "serd_internal.h"
+#include "serd/serd.h"
 
+#include <stdbool.h>
+#include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -44,6 +47,10 @@ serd_env_new(const SerdNode* base_uri)
 void
 serd_env_free(SerdEnv* env)
 {
+	if (!env) {
+		return;
+	}
+
 	for (size_t i = 0; i < env->n_prefixes; ++i) {
 		serd_node_free(&env->prefixes[i].name);
 		serd_node_free(&env->prefixes[i].uri);
@@ -109,9 +116,11 @@ serd_env_add(SerdEnv*        env,
 {
 	SerdPrefix* const prefix = serd_env_find(env, name->buf, name->n_bytes);
 	if (prefix) {
-		SerdNode old_prefix_uri = prefix->uri;
-		prefix->uri = serd_node_copy(uri);
-		serd_node_free(&old_prefix_uri);
+		if (!serd_node_equals(&prefix->uri, uri)) {
+			SerdNode old_prefix_uri = prefix->uri;
+			prefix->uri = serd_node_copy(uri);
+			serd_node_free(&old_prefix_uri);
+		}
 	} else {
 		env->prefixes = (SerdPrefix*)realloc(
 			env->prefixes, (++env->n_prefixes) * sizeof(SerdPrefix));
@@ -205,6 +214,13 @@ serd_env_expand_node(const SerdEnv*  env,
                      const SerdNode* node)
 {
 	switch (node->type) {
+	case SERD_NOTHING:
+	case SERD_LITERAL:
+		break;
+	case SERD_URI: {
+		SerdURI ignored;
+		return serd_node_new_uri_from_node(node, &env->base_uri, &ignored);
+	}
 	case SERD_CURIE: {
 		SerdChunk prefix;
 		SerdChunk suffix;
@@ -218,13 +234,10 @@ serd_env_expand_node(const SerdEnv*  env,
 		ret.n_chars = serd_strlen(buf, NULL, NULL);
 		return ret;
 	}
-	case SERD_URI: {
-		SerdURI ignored;
-		return serd_node_new_uri_from_node(node, &env->base_uri, &ignored);
-	}
-	default:
-		return SERD_NODE_NULL;
+	case SERD_BLANK:
+		break;
 	}
+	return SERD_NODE_NULL;
 }
 
 void
