@@ -1,5 +1,5 @@
 /*
-  Copyright 2011-2016 David Robillard <http://drobilla.net>
+  Copyright 2011-2020 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -14,9 +14,13 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-#include "serd_internal.h"
+#include "string_utils.h"
+
+#include "serd/serd.h"
 
 #include <math.h>
+#include <stdint.h>
+#include <stdlib.h>
 
 void
 serd_free(void* ptr)
@@ -37,6 +41,7 @@ serd_strerror(SerdStatus status)
 	case SERD_ERR_ID_CLASH:   return (const uint8_t*)"Blank node ID clash";
 	case SERD_ERR_BAD_CURIE:  return (const uint8_t*)"Invalid CURIE";
 	case SERD_ERR_INTERNAL:   return (const uint8_t*)"Internal error";
+	default:                  break;
 	}
 	return (const uint8_t*)"Unknown error";  // never reached
 }
@@ -50,6 +55,8 @@ serd_update_flags(const uint8_t c, SerdNodeFlags* const flags)
 		break;
 	case '"':
 		*flags |= SERD_HAS_QUOTE;
+	default:
+		break;
 	}
 }
 
@@ -157,52 +164,3 @@ serd_strtod(const char* str, char** endptr)
 
 	return result * sign;
 }
-
-/**
-   Base64 decoding table.
-   This is indexed by encoded characters and returns the numeric value used
-   for decoding, shifted up by 47 to be in the range of printable ASCII.
-   A '$' is a placeholder for characters not in the base64 alphabet.
-*/
-static const char b64_unmap[] =
-	"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$m$$$ncdefghijkl$$$$$$"
-	"$/0123456789:;<=>?@ABCDEFGH$$$$$$IJKLMNOPQRSTUVWXYZ[\\]^_`ab$$$$"
-	"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"
-	"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$";
-
-static inline uint8_t
-unmap(const uint8_t in)
-{
-	return (uint8_t)(b64_unmap[in] - 47);
-}
-
-/**
-   Decode 4 base64 characters to 3 raw bytes.
-*/
-static inline size_t
-decode_chunk(const uint8_t in[4], uint8_t out[3])
-{
-	out[0] = (uint8_t)(((unmap(in[0]) << 2))        | unmap(in[1]) >> 4);
-	out[1] = (uint8_t)(((unmap(in[1]) << 4) & 0xF0) | unmap(in[2]) >> 2);
-	out[2] = (uint8_t)(((unmap(in[2]) << 6) & 0xC0) | unmap(in[3]));
-	return 1u + (in[2] != '=') + ((in[2] != '=') && (in[3] != '='));
-}
-
-void*
-serd_base64_decode(const uint8_t* str, size_t len, size_t* size)
-{
-	void* buf = malloc((len * 3) / 4 + 2);
-	*size = 0;
-	for (size_t i = 0, j = 0; i < len; j += 3) {
-		uint8_t in[] = "====";
-		size_t  n_in = 0;
-		for (; i < len && n_in < 4; ++n_in) {
-			for (; i < len && !is_base64(str[i]); ++i) {}  // Skip junk
-			in[n_in] = str[i++];
-		}
-		if (n_in > 1) {
-			*size += decode_chunk(in, (uint8_t*)buf + j);
-		}
-	}
-	return buf;
-}
