@@ -1,5 +1,5 @@
 /*
-  Copyright 2011-2017 David Robillard <http://drobilla.net>
+  Copyright 2011-2020 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -14,13 +14,30 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
+#include "byte_sink.h"
 #include "serd_internal.h"
+#include "stack.h"
+#include "string_utils.h"
+#include "uri_utils.h"
+
+#include "serd/serd.h"
 
 #include <assert.h>
+#include <stdarg.h>
+#include <stdbool.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+typedef enum {
+	FIELD_NONE,
+	FIELD_SUBJECT,
+	FIELD_PREDICATE,
+	FIELD_OBJECT,
+	FIELD_GRAPH
+} Field;
+
 typedef struct {
 	SerdNode graph;
 	SerdNode subject;
@@ -69,7 +86,7 @@ static const SepRule rules[] = {
 	{ "[",      1, 0, 1, 1 },
 	{ "]",      1, 1, 0, 0 },
 	{ "(",      1, 0, 0, 0 },
-	{ NULL,     1, 0, 1, 0 },
+	{ NULL,     0, 0, 1, 0 },
 	{ ")",      1, 1, 0, 0 },
 	{ " {",     2, 0, 1, 1 },
 	{ " }",     2, 0, 1, 1 },
@@ -254,6 +271,8 @@ lname_must_escape(const uint8_t c)
 	case '(': case ')': case '*': case '+': case ',': case '/':
 	case ';': case '=': case '?': case '@': case '~':
 		return true;
+	default:
+		break;
 	}
 	return false;
 }
@@ -333,6 +352,7 @@ write_text(SerdWriter* writer, TextContext ctx,
 				switch (in) {
 				case '\b': len += sink("\\b", 2, writer); continue;
 				case '\f': len += sink("\\f", 2, writer); continue;
+				default: break;
 				}
 			}
 		}
@@ -507,7 +527,9 @@ write_uri_node(SerdWriter* const        writer,
 
 	write_sep(writer, SEP_URI_BEGIN);
 	if (writer->style & SERD_STYLE_RESOLVED) {
-		SerdURI in_base_uri, uri, abs_uri;
+		SerdURI in_base_uri;
+		SerdURI uri;
+		SerdURI abs_uri;
 		serd_env_get_base_uri(writer->env, &in_base_uri);
 		serd_uri_parse(node->buf, &uri);
 		serd_uri_resolve(&uri, &in_base_uri, &abs_uri);
@@ -538,9 +560,10 @@ write_curie(SerdWriter* const        writer,
             const Field              field,
             const SerdStatementFlags flags)
 {
-	SerdChunk  prefix;
-	SerdChunk  suffix;
-	SerdStatus st;
+	SerdChunk  prefix = {NULL, 0};
+	SerdChunk  suffix = {NULL, 0};
+	SerdStatus st     = SERD_SUCCESS;
+
 	switch (writer->syntax) {
 	case SERD_NTRIPLES:
 	case SERD_NQUADS:
@@ -619,6 +642,8 @@ write_node(SerdWriter*        writer,
 {
 	bool ret = false;
 	switch (node->type) {
+	case SERD_NOTHING:
+		break;
 	case SERD_LITERAL:
 		ret = write_literal(writer, node, datatype, lang, flags);
 		break;
@@ -630,7 +655,7 @@ write_node(SerdWriter*        writer,
 		break;
 	case SERD_BLANK:
 		ret = write_blank(writer, node, field, flags);
-	default: break;
+		break;
 	}
 	writer->last_sep = SEP_NONE;
 	return ret;
@@ -686,13 +711,13 @@ serd_writer_write_statement(SerdWriter*        writer,
 	}
 
 #define TRY(write_result) \
-	if (!(write_result)) { \
-		return SERD_ERR_UNKNOWN; \
-	}
+	do { \
+		if (!(write_result)) { \
+			return SERD_ERR_UNKNOWN; \
+		} \
+	} while (0)
 
-	switch (writer->syntax) {
-	case SERD_NTRIPLES:
-	case SERD_NQUADS:
+	if (writer->syntax == SERD_NTRIPLES || writer->syntax == SERD_NQUADS) {
 		TRY(write_node(writer, subject, NULL, NULL, FIELD_SUBJECT, flags));
 		sink(" ", 1, writer);
 		TRY(write_node(writer, predicate, NULL, NULL, FIELD_PREDICATE, flags));
@@ -704,8 +729,6 @@ serd_writer_write_statement(SerdWriter*        writer,
 		}
 		sink(" .\n", 3, writer);
 		return SERD_SUCCESS;
-	default:
-		break;
 	}
 
 	if ((graph && !serd_node_equals(graph, &writer->context.graph)) ||
@@ -955,6 +978,10 @@ serd_writer_set_prefix(SerdWriter*     writer,
 void
 serd_writer_free(SerdWriter* writer)
 {
+	if (!writer) {
+		return;
+	}
+
 	serd_writer_finish(writer);
 	serd_stack_free(&writer->anon_stack);
 	free(writer->bprefix);
